
//-------------------------------------------------------------------------------
//Vertex and pixel shader demonstrating ambient, diffuse and specular lighting
// Ambient intensity values are pre baked
//------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------
// Global params provided by the app
//------------------------------------------------------------------------------------------------------
uniform extern float4x4 gWVP;
uniform extern float4x4 gWorldViewIT;
uniform extern float4x4 gWorldView;
uniform extern float4x4 gViewIT;
uniform extern float4 gDiffuseMtrl;
uniform extern float4 gAmbMtrl;
uniform extern float4 gSpecMtrl;
uniform extern float4 gLightCol;
uniform extern float4 gLightDir;
uniform extern texture gTex;

// light intensity
float4 I_a = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient 
float4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
float4 I_s = { 1.0f, 1.0f, 1.0f, 1.0f };    // specular



sampler SimpleSampler = sampler_state
{
	texture = <gTex>;
	MIPFILTER = LINEAR;
    MAGFILTER = LINEAR;
    MINFILTER = Anisotropic;
    MaxAnisotropy = 8;
    AddressU=WRAP;
	AddressV=WRAP;
};


float  n   : MATERIALPOWER = 16; //64.0f;                       
//---------------------------------------------------------------------------------------------------------------------------
// Input channel (vertex shader)
//---------------------------------------------------------------------------------------------------------------------------
struct InputVS
{
	float3 posH : POSITION0;
	float3 Norm : NORMAL;
	float2 uv : TEXCOORD0;
	
};
//---------------------------------------------------------------------------------------------------------------------------
// Output channel (vertex shader)
//---------------------------------------------------------------------------------------------------------------------------
struct OutputVS
{
	float4 posH : POSITION0;
	float3 col: COLOR0;
	float2 uv: TEXCOORD1;
	float3 col_amb: TEXCOORD0;
	float3 col_diff: TEXCOORD2;
	float3 col_spec: TEXCOORD3;
	float3 EV: TEXCOORD4;	
	float3 N: TEXCOORD5;
	float3 E: TEXCOORD6;	
	
};
//---------------------------------------------------------------------------------------------------------------------------
// Vertex shader
//---------------------------------------------------------------------------------------------------------------------------

OutputVS PhongShaderVS(InputVS input)
{
	//Zero out our output
	OutputVS outVS = (OutputVS)0;
	
	//Transform to homogeneous clipspace
	outVS.posH = mul(float4(input.posH, 1.0f), gWVP);
	//eye position 
	outVS.EV = mul(input.posH,gWorldView).xyz;
	//normals
	outVS.N =mul(input.Norm,gWorldViewIT).xyz;
	//eye vector
	outVS.E=normalize(outVS.EV);
		
	outVS.col_amb =  I_a*gAmbMtrl*gLightCol;
	outVS.col_diff = I_d*gDiffuseMtrl*gLightCol;
	outVS.col_spec = I_s*gSpecMtrl*gLightCol;
	
	outVS.uv = input.uv; 
	
	return outVS;
}
//---------------------------------------------------------------------------------------------------------------------------
// Input channel pixel shader
//---------------------------------------------------------------------------------------------------------------------------
struct InputPS
{
	float4 col: COLOR0;
	float2 uv: TEXCOORD1;
	float4 col_amb: TEXCOORD0;
	float4 col_diff: TEXCOORD2;
	float4 col_spec: TEXCOORD3;
	float3 EV: TEXCOORD4;
	float3 N: TEXCOORD5;
	float3 E: TEXCOORD6;	
};
//---------------------------------------------------------------------------------------------------------------------------
// Pixel shader (input channel):output channel
//---------------------------------------------------------------------------------------------------------------------------
float4 PhongShaderPS(InputPS input): COLOR
{
	//transform light vector into viewspace and normalize
	//float4 finalColour=tex2D( SimpleSampler, input.uv );
	//input.N =- normalize(input.N).xyz;
	
	float3 L=-normalize(mul(gLightDir,gWorldView));
	
	//half angle vector used in approximation to specular component
	float3 H=normalize(input.E+L);
	
	//ambient term	
	float4 ambient={0.4, 0.4, 0.4, 1.00f};


	//Calculate diff, sp

	float diff = max(0, dot(input.N,L));
    
	//specular component using Blinns half angle
	//
	float spec = pow(max(0, dot(input.N, H)), n);	
	float4 texColor = tex2D(SimpleSampler, input.uv);	
	
	float4 finalCol =input.col_diff*diff*texColor+  input.col_amb*texColor+ input.col_spec*spec*texColor;  
	finalCol.a=gDiffuseMtrl.a*texColor.a;	
	
	return finalCol;	
}

technique PhongShaderTech
{
	pass P0
	{ 
	    //Lighting       = TRUE;
        SpecularEnable = TRUE;
		vertexShader = compile vs_3_0 PhongShaderVS();
		pixelShader = compile ps_3_0 PhongShaderPS();
		//specify render device states associated with the pass
		FillMode = Solid; //WireFrame;		
		
	}
}
