//-------------------------------------------------------------------------------
//Vertex and pixel shader demonstrating ambient, diffuse and specular lighting
// Ambient intensity values are pre baked
//------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------
// Global params provided by the app
//------------------------------------------------------------------------------------------------------
uniform extern float4x4 gWVP;
uniform extern float4x4 gWorldViewIT;
uniform extern float4x4 gWorldView;
uniform extern float4 gDiffuseMtrl;
uniform extern float4 gAmbMtrl;
uniform extern float4 gSpecMtrl;
uniform extern float4 gLightCol;
uniform extern float4 gLightDir;

// light intensity
float4 I_a = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient 
float4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
float4 I_s = { 1.0f, 1.0f, 1.0f, 1.0f };    // specular


float  n   : MATERIALPOWER = 64.0f;                          // power
//---------------------------------------------------------------------------------------------------------------------------
// Input channel (vertex shader)
//---------------------------------------------------------------------------------------------------------------------------
struct InputVS
{
	float3 posH : POSITION0;
	float3 Norm : NORMAL;
	
};	
//---------------------------------------------------------------------------------------------------------------------------
// Output channel (vertex shader)
//---------------------------------------------------------------------------------------------------------------------------
struct OutputVS
{
	float4 posH : POSITION0;
	float4 col: COLOR0;
	

};
//---------------------------------------------------------------------------------------------------------------------------
// Vertex shader
//---------------------------------------------------------------------------------------------------------------------------

OutputVS SpecAmbDiffVS(InputVS input)
{
	//Zero out our output
	OutputVS outVS = (OutputVS)0;
	
	//Transform to homogeneous clipspace
	outVS.posH = mul(float4(input.posH, 1.0f), gWVP);
		
	float4 col_amb = gAmbMtrl*I_a*gLightCol;
	float4 col_diff = gDiffuseMtrl*I_d*gLightCol;
	float4 col_spec = gSpecMtrl*I_s*gLightCol;
	
	//return output
	//transform light vector into viewspace and normalize
	float4 finalColour=0;
	

    //light direction in camera space
	float3 L=-normalize(mul(gLightDir, gWorldView));
	//eye position 
	float3 EV = mul(input.posH,gWorldView).xyz;
	//normals
	float3 N =mul(input.Norm,gWorldViewIT).xyz;
	//eye vector
	float3 E=normalize(EV);
	
	//half angle vector used in approximation to specular component
	
	float3 H=normalize(E+L);
	

	//Calculate diff, specular and ambient components of lighht	
	//ambient term	
	float4 ambient={0.35, 0.4, 1.0, 1.00f};

	//diffuse term
	float diff = max(0, dot(N,L));
    
	//specular component using half angle
	float spec = pow(max(0, dot(N, H)), n);	
	

	outVS.col.rgb = ambient*col_amb + diff*col_diff+ spec*col_spec;
	outVS.col.a=1.0;

	//return 
	return outVS;
}
//---------------------------------------------------------------------------------------------------------------------------
// Input channel pixel shader
//---------------------------------------------------------------------------------------------------------------------------
struct InputPS
{
	float4 col: COLOR0;



};
//---------------------------------------------------------------------------------------------------------------------------
// Pixel shader (input channel):output channel
//---------------------------------------------------------------------------------------------------------------------------
float4 SpecAmbDiffPS(InputPS input): COLOR
{
	return input.col;	
}

technique SpecAmbDiffTech
{
	pass P0
	{ 
	    Lighting       = TRUE;
        SpecularEnable = TRUE;
		vertexShader = compile vs_2_0 SpecAmbDiffVS();
		pixelShader = compile ps_2_0 SpecAmbDiffPS();
		//specify render device states associated with the pass
		FillMode = Solid;// WireFrame;
		ShadeMode = Gouraud;
	}
}


